#!/usr/bin/env python
import paths as pt
from tools import file_reader, file_writer
from tools import preprocessor
from utility import embedder
import pandas as pd
import numpy as np
from pathlib import Path
import yaml

def main():
    make_complete_count()
    make_compliance_count()
    make_fall_count()
    make_risk()
    
def make_complete_count():
    with open(Path.joinpath(pt.CONFIGS_DIR, "complete_emb.yaml"), 'r') as stream:
        settings = yaml.safe_load(stream)
    case = 'Complete'
    ats = {str(i)+'Ats':str for i in range(1, settings['ats_resolution']+1)}
    df = file_reader.read_csv(pt.PROCESSED_DATA_DIR,
                              f'complete.csv',
                              converters=ats)
    
    cols_ats = [str(i)+'Ats' for i in range(1, settings['ats_resolution']+1)]
    unique_ats = [df[f'{i}Ats'].unique() for i in range(1, settings['ats_resolution']+1)]
    unique_ats = list(set(np.concatenate(unique_ats)))
    
    df_ats = preprocessor.extract_cat_count(df, unique_ats, cols_ats, 'Ats_')
    df_ats = df_ats.drop(['Ats_0'], axis=1)
    
    df = df.drop(cols_ats, axis=1)
    df = pd.concat([df.drop(case, axis=1), df_ats, df[[case]]], axis=1)

    file_writer.write_csv(df, pt.PROCESSED_DATA_DIR, 'complete_count.csv')
    
def make_compliance_count():
    with open(Path.joinpath(pt.CONFIGS_DIR, "compliance_emb.yaml"), 'r') as stream:
        settings = yaml.safe_load(stream)
    case = 'Compliance'
    ats = {str(i)+'Ats':str for i in range(1, settings['ats_resolution']+1)}
    df = file_reader.read_csv(pt.PROCESSED_DATA_DIR,
                              f'compliance.csv',
                              converters=ats)
    cols_ats = [str(i)+'Ats' for i in range(1, settings['ats_resolution']+1)]
    unique_ats = [df[f'{i}Ats'].unique() for i in range(1, settings['ats_resolution']+1)]
    unique_ats = list(set(np.concatenate(unique_ats)))
    
    df_ats = preprocessor.extract_cat_count(df, unique_ats, cols_ats, 'Ats_')
    df_ats = df_ats.drop(['Ats_0'], axis=1)
    
    df = df.drop(cols_ats, axis=1)
    df = pd.concat([df.drop(case, axis=1), df_ats, df[[case]]], axis=1)
        
    file_writer.write_csv(df, pt.PROCESSED_DATA_DIR, 'compliance_count.csv')

def make_fall_count():
    with open(Path.joinpath(pt.CONFIGS_DIR, "fall_emb.yaml"), 'r') as stream:
        settings = yaml.safe_load(stream)
    case = 'Fall'
    ats = {str(i)+'Ats':str for i in range(1, settings['ats_resolution']+1)}
    df = file_reader.read_csv(pt.PROCESSED_DATA_DIR,
                              f'fall.csv',
                              converters=ats)
    
    cols_ats = [str(i)+'Ats' for i in range(1, settings['ats_resolution']+1)]
    unique_ats = [df[f'{i}Ats'].unique() for i in range(1, settings['ats_resolution']+1)]
    unique_ats = list(set(np.concatenate(unique_ats)))
    
    df_ats = preprocessor.extract_cat_count(df, unique_ats, cols_ats, 'Ats_')
    df_ats = df_ats.drop(['Ats_0'], axis=1)
    
    df = df.drop(cols_ats, axis=1)
    df = pd.concat([df.drop(case, axis=1), df_ats, df[[case]]], axis=1)
        
    file_writer.write_csv(df, pt.PROCESSED_DATA_DIR, 'fall_count.csv')

def make_risk():
    with open(Path.joinpath(pt.CONFIGS_DIR, "risk_emb.yaml"), 'r') as stream:
        settings = yaml.safe_load(stream)
    case = 'Fall'
    ex = {str(i)+'Ex':str for i in range(1, settings['ex_resolution']+1)}
    ats = {str(i)+'Ats':str for i in range(1, settings['ats_resolution']+1)}
    converters = {**ex, **ats}
    df = file_reader.read_csv(pt.PROCESSED_DATA_DIR,
                              f'fall_test.csv',
                              converters=converters)
    
    num_cols = embedder.get_numerical_cols(df, case)
    
    # Extract exercises
    cols_ex = [str(i)+'Ex' for i in range(1, settings['ex_resolution']+1)]
    unique_ex = [df[f'{i}Ex'].unique() for i in range(1, settings['ex_resolution']+1)]
    unique_ex = list(set(np.concatenate(unique_ex)))
    df_ex = preprocessor.extract_cat_count(df, unique_ex, cols_ex, 'Ex_')
    
    # Extract ats
    cols_ats = [str(i)+'Ats' for i in range(1, settings['ats_resolution']+1)]
    unique_ats = [df[f'{i}Ats'].unique() for i in range(1, settings['ats_resolution']+1)]
    unique_ats = list(set(np.concatenate(unique_ats)))
    df_ats = preprocessor.extract_cat_count(df, unique_ats, cols_ats, 'Ats_')

    # Merge dataframes
    df = pd.concat([df, df_ex, df_ats], axis=1)
    ex_columns = ['Ex_' + ex for ex in unique_ex]
    ats_columns = ['Ats_' + ats for ats in unique_ats]
    df = df[num_cols + ex_columns + ats_columns + [case]]
    df = df.drop(['Ex_0', 'Ats_0'], axis=1)
    file_writer.write_csv(df, pt.PROCESSED_DATA_DIR, 'fall_test_count.csv')

if __name__ == "__main__":
    main()